package org.colcon.web.model;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.IArgList;
import org.colcon.burocrat.api.model.IFlag;
import org.colcon.burocrat.api.model.IItem;
import org.colcon.burocrat.api.model.IItemType;
import org.colcon.burocrat.api.utils.ArrayUtil;
import org.colcon.burocrat.api.utils.StringUtil;
import org.colcon.burocrat.model.Arg;
import org.colcon.burocrat.model.ArgList;
import org.colcon.burocrat.model.Constants;
import org.colcon.burocrat.model.ItemType;
import org.colcon.burocrat.model.Model;

public enum AppItemType implements IItemType
{
	USER("users", "a user's personal information", ItemType.AGENT, new ArgList(Arg.EMAIL, Arg.ID, Arg.PASSWORD), new IItemType[0], new IFlag[0])
	{
		private final int CONFIRMATION_CODE_LENGTH = 32;
		
		public void init(IItem instance)
		{
			super.init(instance);
			instance.setArg(Arg.PARENT, plural());
			
			String confirmationCode = StringUtil.randomString(CONFIRMATION_CODE_LENGTH);
			instance.setArg(Arg.CONFIRMATION_CODE, confirmationCode);
			
			if(!instance.hasArg(Arg.EMAIL_CONFIRMED))
				instance.setArg(Arg.EMAIL_CONFIRMED, Boolean.toString(false));
		}
	},
	
	GROUP("groups", "A group of agents (users or other groups) with some common purpose, contains all group data (votes, laws, titles, etc.)", 
			ItemType.AGENT, new ArgList(Arg.ID, Arg.SUBJECT, AppArg.OPEN), new IItemType[]{}, new IFlag[0])
	{
		public void init(IItem instance)
		{
			super.init(instance);
			instance.setArg(Arg.PARENT, plural());
		}
	},
	
	BALLOT("ballots", "A string value submitted by an agent to a vote", 
			ItemType.LINK, new ArgList(Arg.VALUE), new IItemType[0], new IFlag[0]),
			
	VOTE("votes", "A decision to be made collectively by the group based on input from its members, which may result in a command to be executed by the server",
			ItemType.CONTAINER, new ArgList(Arg.SUBJECT, AppArg.CHOICES), new IItemType[0], new IFlag[0]),

;
	
	private String plural, description;
	private IItemType parent;
	
	private IArgList args;
	private IItemType[] containerTypes;
	private Set<IFlag> flags;
	
	private AppItemType(String plural, String description, IItemType parent, IArgList args, IItemType[] containers, IFlag[] flags)
	{
		this.plural = plural;
		this.description = description;
		this.parent = parent;
		this.args = args;
		this.containerTypes = containers;
		
		this.flags = new HashSet<IFlag>();
		this.flags.addAll(Arrays.asList(flags));
	}
	
	public String plural()
	{
		return plural;
	}
	
	public String description()
	{
		return description;
	}
	
	public IItemType parent()
	{
		return parent;
	}
	
	public IArgList args()
	{
		return args;
	}
	
	public IArgList getAllArgs()
	{
		if(parent != null)
			return parent.getAllArgs().append(args());
		
		return args();
	}
	
	public IItemType[] containerTypes()
	{
		return containerTypes;
	}
	
	public IItemType[] getAllContainerTypes()
	{
		if(parent != null)
			ArrayUtil.append(parent.getAllContainerTypes(), containerTypes());
		
		return containerTypes();
	}
	
	public IItem[] getInitItems(IItem instance) 
	{
		return new IItem[0];
	}
	
	public IItem[] getAllInitItems(IItem instance) 
	{
		if(parent != null)
			ArrayUtil.append(parent.getAllInitItems(instance), getInitItems(instance));
		
		return new IItem[0];
	}
	
	public IFlag[] flags()
	{
		if(parent() != null)
			return ArrayUtil.append(parent().flags(), flags.toArray(new IFlag[0]));
		
		return flags.toArray(new IFlag[0]);
	}
	
	public boolean hasFlag(IFlag flag)
	{
		return flags.contains(flag);
	}

	public void init(IItem instance) 
	{
		if(parent != null)
			parent.init(instance);
		
		if(instance.getId() == null)	
			instance.setArg(Arg.ID, generateId(instance));
		
		if(instance.getParent() == null)
			instance.setArg(Arg.PARENT, Constants.ROOT);
	}
	
	public String generateId(IItem instance)
	{
		if(parent != null)
			return parent.generateId(instance);
		
		return null;
	}
	
	public Map<IArg, String> withArgs(Object... values)
	{
		Map<IArg, String> map = getAllArgs().withArgs(values);
		
		map.put(Arg.TYPE, name());
		
		return map;
	}
	
	public String getInstancePath(String id)
	{
		return Model.instance.parser().composePath(plural(), id);
	}
	
	public static IItemType[] getValues()
	{
		return (IItemType[])values();
	}
}


