package org.colcon.burocrat.client;

import java.util.HashMap;
import java.util.Map;

import org.colcon.burocrat.api.client.IClient;
import org.colcon.burocrat.api.client.IFieldTypeImpl;
import org.colcon.burocrat.api.client.IItemPageImpl;
import org.colcon.burocrat.api.client.IPageImpl;
import org.colcon.burocrat.api.client.IWidgets;
import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.ICommand;
import org.colcon.burocrat.api.model.IItem;
import org.colcon.burocrat.api.model.IResponse;
import org.colcon.burocrat.api.utils.EnumUtil;
import org.colcon.burocrat.client.pages.ItemRequestPage;
import org.colcon.burocrat.client.ui.HeaderPanel;
import org.colcon.burocrat.model.Arg;
import org.colcon.burocrat.model.Cmd;
import org.colcon.burocrat.model.Constants;
import org.colcon.burocrat.model.ItemType;
import org.colcon.burocrat.model.Model;
import org.colcon.burocrat.model.Command;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.History;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.Widget;

/**
 * The main class of the Burocrat client
 * 
 * @author Dan Vegeto
 *
 */
public class Client implements IClient, ValueChangeHandler<String>
{	
	public static final String
	
		URL_ARGS_SEPARATOR = "?",
		URL_ARGS_SEPARATOR_REGEX = "\\?";
	
	private ICommand GET_USER_COMMAND;
	
	private String title;
	
	private Map<String, IPageImpl> pageImpls;
	private Map<String, IItemPageImpl> itemPageImpls;
	private Map<String, IFieldTypeImpl> fieldTypeImpls;
	
	private IWidgets widgets;
	private Panel rootPanel;
	private BurocratServiceAsync service;

	private HeaderPanel header;
	
	public Client(String title, IPageImpl[] pageImplsArray, IItemPageImpl[] itemPageArray, IFieldTypeImpl[] fieldTypeImplsArray, IWidgets widgets, Panel rootPanel, BurocratServiceAsync service)
	{
		this.title = title;

		pageImpls = new HashMap<String, IPageImpl>();
		itemPageImpls = new HashMap<String, IItemPageImpl>();
		fieldTypeImpls = new HashMap<String, IFieldTypeImpl>();
		
		EnumUtil.putAll(pageImpls, pageImplsArray);
		EnumUtil.putAll(itemPageImpls, itemPageArray);
		EnumUtil.putAll(fieldTypeImpls, fieldTypeImplsArray);
		
		this.widgets = widgets;
		this.rootPanel = rootPanel;
		this.service = service;
		
		History.addValueChangeHandler(this);
	}
	
	public IWidgets widgets()
	{
		return widgets;
	}
	
	public IPageImpl[] getPageImpls()
	{
		return pageImpls.values().toArray(new IPageImpl[0]);
	}
	
	public IItemPageImpl[] getItemPageImpls()
	{
		return itemPageImpls.values().toArray(new IItemPageImpl[0]);
	}
	
	public IFieldTypeImpl[] getFieldTypeImpls()
	{
		return fieldTypeImpls.values().toArray(new IFieldTypeImpl[0]);
	}
	
	public IPageImpl getPageImpl(String name)
	{
		return pageImpls.get(name.toUpperCase());
	}
	
	public IItemPageImpl getItemPageImpl(String name)
	{
		return itemPageImpls.get(name.toUpperCase());
	}
	
	public IFieldTypeImpl getFieldTypeImpl(String name)
	{
		return fieldTypeImpls.get(name.toUpperCase());
	}
	
	public boolean isLoggedIn()
	{
		return 
				getUserId() != null &&
				getUserPassword() != null;
	}
	
	public String getUserId()
	{
		return Cookies.getCookie(BurocratCookie.ID.name());
	}
	
	public String getUserPassword()
	{
		return Cookies.getCookie(BurocratCookie.PASSWORD.name());
	}
	
	public void login(String id, String password)
	{
		Cookies.setCookie(BurocratCookie.ID.name(), id);
		Cookies.setCookie(BurocratCookie.PASSWORD.name(), password);
		
		GET_USER_COMMAND = new Command(Arg.CMD.put(Cmd.GET), Arg.PATH.put(ItemType.USER.getInstancePath(id)));
		
		service.sendRequest(id, password, GET_USER_COMMAND.toString(), new AsyncCallback<String>()
		{
			public void onFailure(Throwable caught)
			{
				//TODO: handle this failure
			}
		
			public void onSuccess(String result)
			{
				Cookies.setCookie(BurocratCookie.USER.name(), result);
			}
		});
		
		refresh();
	}
	
	public void logout()
	{
		Cookies.removeCookie(BurocratCookie.ID.name());
		Cookies.removeCookie(BurocratCookie.PASSWORD.name());
		Cookies.removeCookie(BurocratCookie.USER.name());
		
		refresh();
	}
	
	public String getURL()
	{
		return History.getToken();
	}
	
	public String getPath()
	{
		return getPath(getURL());
	}
	
	private String getPath(String url)
	{	
		if(url == null)
			return null;
		if(url.isEmpty())
			return "";
		
		String[] array = url.split("\\?");
		
		if(array.length > 0)
			return array[0];

		return url;
	}
	
	public Map<IArg, String> getArgs()
	{
		return getArgs(getURL());
	}
	
	private Map<IArg, String> getArgs(String url)
	{
		String[] array = url.split(URL_ARGS_SEPARATOR_REGEX);
	
		if(array.length > 1)
			return Model.instance.parser().parseArgs(array[1]);
	
		return new HashMap<IArg, String>();
	}
	
	public void setArgs(Map<IArg, String> map)
	{
		if(map == null || map.isEmpty())
			return;
		
		String args = Model.instance.parser().composeArgs(map);
		
		String url = getPath() + URL_ARGS_SEPARATOR + args;
		
		History.newItem(url, false);
	}
	
	public void refresh()
	{	
		show(getURL());
	}
	
	public void sendRequest(ICommand command, final AsyncCallback<IResponse> callback)
	{
		String id = getUserId();
		String password = getUserPassword();
		String commandStr = command.toString();
		
		if(!isLoggedIn())
		{
			id = Constants.GUEST;
			password = Constants.GUEST_PASSWORD;
		}
		
		service.sendRequest(id, password, commandStr, new AsyncCallback<String>()
		{
			public void onFailure(Throwable caught) 
			{
				callback.onFailure(caught);
			}

			public void onSuccess(String result) 
			{
				IResponse response = Model.instance.parser().parseResponse(result);
				callback.onSuccess(response);
			}
		});
	}
	
	public void onValueChange(ValueChangeEvent<String> event) 
	{
		show(event.getValue());
	}
	
	public void show(String url)
	{
		History.newItem(url);
		
		String path = getPath();
		if(path != null && !path.isEmpty())
			Window.setTitle(title + ' ' + path);
		else
			Window.setTitle(title);
		
		show(buildPage(url));
	}
	
	public void show(Widget widget)
	{
		rootPanel.clear();
		rootPanel.add(widget);
	}
	
	private Widget buildPage(String url)
	{
		if(url == null || url.isEmpty())
			return buildPage(PageImpl.HOME.name());
	
		String path = getPath(url);
		
		IPageImpl pageEnum = getPageImpl(path);
		
		if(pageEnum != null)
			return pageEnum.buildPage(this);
		
		return new ItemRequestPage(this, path);
	}

	public IClient addHeader(HeaderPanel h)
	{
		header = h;
		return this;
	}

	public HeaderPanel getHeader()
	{
		if(header != null)
			return header;
		
		return new HeaderPanel(this);
	}

	public IItem getUser()
	{
		if(Cookies.getCookie(BurocratCookie.USER.name()) != null)
			return Model.instance.parser().parseItem(Cookies.getCookie(BurocratCookie.USER.name()));
		
		return null;
	}
}
