package org.colcon.burocrat.client.form;

import java.util.ArrayList;

import org.colcon.burocrat.api.client.IClient;
import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.IChoice;
import org.colcon.burocrat.api.model.IResponse;
import org.colcon.burocrat.client.BurocratField;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.RadioButton;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * @author Jesse Freitas, 2013
 * 
 *         RadioChoiceForm is an BurocratField widget available in the MRCWS
 *         project. This widget acts as a dynamic radio button list of choices
 *         to select from as an alternative way of displaying choices from a
 *         drop down menu. The widget can dynamically place the items into
 *         columns to make the most of screen real-estate.
 */
public class RadioSelectionField extends BurocratField implements ClickHandler
{

	private ArrayList<RadioButton> items = new ArrayList<RadioButton>();
	private HorizontalPanel root = new HorizontalPanel();
	private ArrayList<VerticalPanel> columns = new ArrayList<VerticalPanel>();
	private int column_size;
	private IChoice[] choices;

	public static final int NUM_COLUMNS = 0;
	public static final int NUM_ITEMS = 1;

	public RadioSelectionField(IClient client, IArg arg)
	{
		this(client, arg, arg.choices(), 1);
	}
	
	/**
	 * Constructor: Takes an array of IChoice references and places them in a
	 * default 3-4 column layout.
	 * 
	 * @param choices
	 *            - An array of IChoice references to be placed in the displayed
	 *            list of choices
	 * @param n
	 *            - the number of columns to display the list.
	 */
	public RadioSelectionField(IClient client, IArg arg, IChoice[] choices,
			int n)
	{
		super(client, arg);
		column_size = n;
		this.choices = choices;
	}

	/**
	 * Retrieves the selected value of the list or {@code null} if no value is
	 * selected.
	 * 
	 * @return A {@code String} value which is the display name in the list.
	 * @see org.colcon.burocrat.client.BurocratField#getValue()
	 */
	@Override
	public String getValue()
	{
		for (RadioButton option : items)
		{
			if (option.getValue())
			{
				return option.getText();
			}
		}
		return null;
	}

	/**
	 * Sets the value of the form if the value exists.
	 * 
	 * @param - text The name of the item to select
	 * @return - {@code true} if a value was selected, {@code false} otherwise.
	 * @see org.colcon.burocrat.client.BurocratField#setValue(java.lang.String)
	 */
	@Override
	public void setValue(String value)
	{
		for (RadioButton option : items)
		{
			if (option.getText().equals(value))
			{
				option.setValue(true);
				return;
			}
		}
		return;
	}
	
	public void onClick(ClickEvent event)
	{
		getChangeHandler().onChange(null);
	}

	/**
	 * Layout Radio choices.
	 * 
	 * @param The
	 *            {@code IResponse} data received from the Burocrat Server
	 * @see org.colcon.burocrat.client.ui.BurocratPanel#init(org.colcon.burocrat.api.model.IResponse)
	 */
	@Override
	public void init(IResponse data)
	{
		for (IChoice item : choices)
		{
			RadioButton button = new RadioButton(this.hashCode() + "",
					item.name());
			button.setTitle(item.description());
			button.addClickHandler(this);
			items.add(button);
		}
		int row = 0;
		columns.add(new VerticalPanel());
		for (RadioButton option : items)
		{
			if (row == column_size)
			{
				row = 0;
				columns.add(new VerticalPanel());
			}
			columns.get(columns.size() - 1).add(option);
			row++;
		}
		for (VerticalPanel column : columns)
		{
			root.add(column);
		}
		add(root);
	}
}
