package org.colcon.burocrat.server;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.colcon.burocrat.api.server.IDatabase;
import org.colcon.burocrat.model.Arg;
import org.colcon.burocrat.model.Model;

import com.google.appengine.api.datastore.DatastoreService;
import com.google.appengine.api.datastore.DatastoreServiceFactory;
import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.EntityNotFoundException;
import com.google.appengine.api.datastore.FetchOptions;
import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;
import com.google.appengine.api.datastore.Query;
import com.google.appengine.api.datastore.Text;
import com.google.appengine.api.datastore.Query.FilterOperator;
import com.google.appengine.api.datastore.Query.SortDirection;

/**
 * 
 * This is the interface between the Burocrat server and the GWT Datastore
 * 
 * @author Dan Vegeto
 *
 */
public class Database implements IDatabase
{
	private DatastoreService datastore;
	
	private static final String ENTITY_KIND = "item";
	private static final int STRING_CHAR_LIMIT = 500;

	public void init()
	{
		if(datastore == null)
			datastore = DatastoreServiceFactory.getDatastoreService();
	}
	
	public Map<String, String> get(String path) 
	{
		Entity entity = getEntity(path);
		
		if(entity == null)
			return null;
		
		return objectsToStrings(entity.getProperties());
	}

	public void post(Map<String, String> item) 
	{
		String parent = item.get(Arg.PARENT.name());
		String id = item.get(Arg.ID.name());
		
		String path = Model.instance.parser().composePath(parent, id);
		
		Key key = createKey(path);
		
		if(key == null || get(path) != null)
			return;
		
		Entity entity = new Entity(key);
		
		Map<String, Object> map = stringsToObjects(item);
		
		for(String str : map.keySet())
			entity.setProperty(str, map.get(str));
		
		datastore.put(entity);
	}

	public void put(Map<String, String> item) 
	{
		String parent = item.get(Arg.PARENT.name());
		String id = item.get(Arg.ID.name());
		String path = Model.instance.parser().composePath(parent, id);
		
		Key key = createKey(path);
		
		if(key == null || get(path) == null)
			return;
		
		Entity entity = new Entity(key);
		
		Map<String, Object> map = stringsToObjects(item);
		for(String str : map.keySet())
			entity.setProperty(str, map.get(str));
		
		datastore.put(entity);
	}

	public void delete(String path) 
	{
		Key key = createKey(path);
		
		if(key == null || get(path) == null)
			return;
		
		datastore.delete(key);
	}

	public List<Map<String, String>> query(String parent, int limit, int offset, String sort, boolean ascending, Map<String, String> filters) 
	{
		List<Entity> entities = queryEntities(parent, limit, offset, sort, ascending, filters);
		
		List<Map<String, String>> list = new ArrayList<Map<String, String>>();
		
		for(Entity entity : entities)
			list.add(objectsToStrings(entity.getProperties()));
		
		return list;
	}
	
	public List<Entity> queryEntities(String parent, int limit, int offset, String sort, boolean ascending, Map<String, String> filter)
	{
		Query query;
		
		// no ancestor key for multi-parent queries
		if(parent == null || Model.instance.parser().parseArgArray(parent).length > 1)
			query = new Query("item");
		else
			query = new Query("item", createKey(parent));
		
		if(sort != null)
		{
			if(ascending)
				query.addSort(sort, SortDirection.ASCENDING);
			else
				query.addSort(sort, SortDirection.DESCENDING);
		}
		
		if(filter == null)
			filter = new LinkedHashMap<String, String>();
		
		if(parent != null)
			filter.put(Arg.PARENT.name(), parent);
		
		List<Query.Filter> andList = new LinkedList<Query.Filter>();
		List<Query.Filter> orList = new LinkedList<Query.Filter>();
		
		for(String arg : filter.keySet())
		{
			orList.clear();
			
			String value = filter.get(arg);
			
			if(value == null)
				continue;
			
			String[] valueArray = Model.instance.parser().parseArgArray(value);
			
			for(String str : valueArray)
				orList.add(new Query.FilterPredicate(arg.toString(), FilterOperator.EQUAL, str));
			
			if(orList.size() == 1)
				andList.add(orList.get(0));
			else
				andList.add(Query.CompositeFilterOperator.or(orList));
		}
	
		if(andList.size() == 1)
			query.setFilter(andList.get(0));
		else if(andList.size() > 1)
			query.setFilter(Query.CompositeFilterOperator.and(andList));
		
		return datastore.prepare(query).asList(FetchOptions.Builder.withLimit(limit).offset(offset));
	}
	
	private Entity getEntity(String path)
	{
		Key key = createKey(path);
		
		try 
		{
			return datastore.get(key);
		} 
		catch (EntityNotFoundException e) 
		{
			return null;
		}
	}
	
	private Key createKey(String path)
	{
		if(path == null)
			return null;

		String[] array = Model.instance.parser().parsePath(path);
		
		Key key = KeyFactory.createKey(ENTITY_KIND, array[0]);
		
		for(int i = 1; i < array.length; i++)
			key = KeyFactory.createKey(key, ENTITY_KIND, array[i]);
		
		return key;
	}
	
	private Map<String, String> objectsToStrings(Map<String, Object> objMap)
	{
		Map<String, String> strMap = new LinkedHashMap<String, String>();
		
		for(String str : objMap.keySet())
			strMap.put(str, objMap.get(str).toString());
		
		return strMap;
	}
	
	private Map<String, Object> stringsToObjects(Map<String, String> strMap)
	{
		Map<String, Object> objMap = new LinkedHashMap<String, Object>();
		
		for(String str : strMap.keySet())
		{
			String value = strMap.get(str);
			
			if(value == null)
				continue;
			
			if(value.length() < STRING_CHAR_LIMIT)
				objMap.put(str, value);
			else
				objMap.put(str, new Text(value));
		}
		
		return objMap;
	}
}
